// BoltzmannSampler.cpp: this file is part of the REGAL project.
//
// REGAL : Random and Exhaustive Generators for Automata - Library
//
// Copyright (C) 2007 Julien DAVID.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// The complete GNU General Public Licence Notice can be found as the
// `COPYING' file in the root directory.
//
//
#include "BoltzmannSampler.hpp"

namespace regal{
  
  /**
     Mathematical function that randomly picks a value, according to DzetaK
     This value is used to create a partition.
     @param DzetaK parameters the randomness, according to the partition's parameters.
     @Return the generated value. 
  */
  // int BoltzmannSampler::PoissonLaw(double DzetaK){
  //     //int k=0;
  //     //double p=1/exp(DzetaK);
  //     int k=1;
  //     p=DzetaK/(expl(DzetaK)-1);
  //     double dice=(rand())/(double)RAND_MAX;
  //     while(dice>=p){
  //       dice-=p;
  //       k++;
  //       p=DzetaK*p/k;
  //     }
  //     return k;
  //   }
  
  
  
  /**
     Compute DzetaK, a constant which is used to generate partitions.
     In the automaton case, K is the alphabet's size
  */
  void BoltzmannSampler::computeDzetaK(){
    long double W0=0.,tmp;
    long double z=-alphaSize*expl(-(long double)alphaSize);
    long double facti=1;
    long double precision=1/powl(10,20);
    for(int i=1;i;i++){
      facti*=i;
      tmp=pow(-i,i-1)*pow(z,i)/facti;
      W0+=tmp;
      if(tmp<precision&&tmp>-precision)
	break;
    }
    dzetaK=W0+alphaSize;
    //printf("%.20Lf\n",dzetaK);
  } 
  
  
  /**
     Creates a partition, guaranteeing an equiprobability among the results.
     @Return a partition
  */
  int * BoltzmannSampler::sample(){
    int counter=0,i;
    rejectCounter=0;
    int max=autSize*alphaSize;
#ifdef __REJECTFLAG
    std::cout<<"Entree dans le sampler"<<std::endl;
#endif    

    while(counter!=max){
      for(i=0,counter=0;i<autSize;i++){
	//partition[i]=PoissonLaw(dzetaK);
	partition[i]=nzpl->randomValue();
	counter+=partition[i];
	if(counter>max)
	  break;
      }
      rejectCounter++;
    }

    
        
#ifdef __REJECTFLAG
    std::cout<<"Nombre de rejet : "<<rejectCounter<<std::endl;
#endif    

    return partition;
  }

}
