//    This file is part of RoadColoringImpl.
//
//    Copyright (C) 2008 Florian SIKORA sikora@univ-mlv.fr
//    http://igm.univ-mlv.fr/~fsikora/index.php?p=r
//
//    RoadColoringImpl is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    RoadColoringImpl is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    The complete GNU General Public Licence Notice can be found as the
//    `COPYING' file in the root directory.


package fr.umlv.roadcoloring.graph;

/**
 * Datas informations for one node
 * 
 * @author fsikora
 *
 */

public class NodeData {
	private Circle circle;		//reached circle
	private int level;			//level of the node, distance of the circle
	private int rootAbsolute;	//root of the tree
	//private int rootCircle;
	
	public NodeData(Circle circle, int level, int rootAbsolute) {
		this.circle = circle;
		this.level = level;
		this.rootAbsolute = rootAbsolute;
	}

	/**
	 * Get the reached circle
	 * (it's own if node in on the circle, the reached circle of tree else)
	 * @return a Circle
	 */
	public Circle getCircle() {
		return circle;
	}

	/**
	 * return the circle's distance of the node (0 if on the circle)
	 * @return the distance of the node to the circle
	 */
	public int getLevel() {
		return level;
	}

	/**
	 * get the number of the node root
	 * @return the number of the tree's root
	 */
	public int getRootAbsolute() {
		return rootAbsolute;
	}
	
	@Override
	public String toString() {
		return "Circle #"+circle.getCircleNumber() + " level : " + level + " root : " + rootAbsolute;
	}
}
