//    This file is part of RoadColoringImpl.
//
//    Copyright (C) 2008 Florian SIKORA sikora@univ-mlv.fr
//    http://igm.univ-mlv.fr/~fsikora/index.php?p=r
//
//    RoadColoringImpl is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    RoadColoringImpl is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    The complete GNU General Public Licence Notice can be found as the
//    `COPYING' file in the root directory.


package fr.umlv.roadcoloring.main;

import java.io.IOException;

import fr.umlv.roadcoloring.graph.Graph;
import fr.umlv.roadcoloring.graph.Pairs;

/**
 * Main class for launch the algorithm
 * @author fsikora
 *
 */

public class Main {
	
	/**
	 * print usage
	 */
	private static void usage() {
		System.err.println("Usage : \n" + " * Main -n n [generator]\n\t with n graph's node \n\t generator an optional program who gives a graph with the correct syntax (else use random)");
		System.err.println(" * Main -f filename\n\t with filename a file with a graph with the correct syntax");
		System.err.println("\nThe correct syntax for a graph is : \nn\ni r b\n\nwith i number of the node, r the node reached by red, b the node reached by blue ");
	}
	
	
	/**
	 * launch
	 * usage : -n n [generator] for generate a graph of n nodes
	 * -f filename, generate a graph from a file
	 * @param args the args
	 * @throws InterruptedException
	 */
	public static void main(String[] args) throws InterruptedException {
		if(args.length < 2 || args.length > 3) {
			System.err.println("Incorrect number of arguments");
			usage();
			System.exit(1);
		}
		
		Graph g = null;
		
		if ("-n".equals(args[0])) {
			try {			
				int n = Integer.valueOf(args[1]);
				
				if (args.length == 2) {
					g = new Graph(n, null);
				}
				else {
					g = new Graph(n, args[2]);
				}
			}
			catch (NumberFormatException e) {
				System.err.println("A number is needed, not " + args[1]);
				usage();
				System.exit(1);
			} catch (IOException e) {
				System.err.println(args[2] + " unexecutable");
				e.printStackTrace();
				usage();
				System.exit(1);	
			}
		}
		else if ("-f".equals(args[0])) {
			g = new Graph(args[1]);
		}
		else {
			System.err.println(args[0] + " is an unknow option");
			usage();
			System.exit(1);
		}
		
		System.err.println(g);		
		
		//print graph before roadcoloring
		//g.dotty();
		//Thread.sleep(1000);
		
		g.coloring();
		
		//System.out.println("sync word = " + new Pairs(g).getSynchronizedWord());
		System.out.println("\nFinal graph\n" + g);
		String syncWord = new Pairs(g).getSynchronizedWord();
		if(!g.verifySyncWord(syncWord)) throw new AssertionError(syncWord + " not a syncword");
		
		//print graph synchronized
		g.dotty();
	}
}
